@@ -8,7 +8,7 @@
 #   The GNU General Public License, Version 3, June 2007
 # 
 package Games::Pandemic;
-our $VERSION = '0.5.0';
+our $VERSION = '0.6.0';
 
 # ABSTRACT: cooperative pandemic board game
 
@@ -43,20 +43,23 @@ has config => (
 );
 
 has map => (
-    is  => 'rw',
-    isa => 'Games::Pandemic::Map',
+    is      => 'rw',
+    isa     => 'Games::Pandemic::Map',
+    clearer => 'clear_map',
 );
 
 # player cards deck
 has cards => (
-    is  => 'rw',
-    isa => 'Games::Pandemic::Deck',
+    is      => 'rw',
+    isa     => 'Games::Pandemic::Deck',
+    clearer => 'clear_cards_deck',
 );
 
 # infection cards deck
 has infection => (
-    is  => 'rw',
-    isa => 'Games::Pandemic::Deck',
+    is      => 'rw',
+    isa     => 'Games::Pandemic::Deck',
+    clearer => 'clear_infection_deck',
 );
 
 # current players
@@ -69,6 +72,7 @@ has _players => (
     provides   => {
         elements => 'all_players',       # my @p = $game->all_players;
         push     => 'add_player',        # $game->add_player( $player );
+        clear    => 'clear_players',
     }
 );
 # list of players waiting for their turn
@@ -81,12 +85,28 @@ has _players_in_turn => (
     provides   => {
         push     => 'reinit_players',    # $game->reinit_players( $player );
         shift    => 'next_player',       # my $p = $game->next_player;
+        clear    => 'clear_players_in_turn',
     }
 );
-has curplayer => ( is=>'rw', isa=>'Games::Pandemic::Player', weak_ref=>1 );
+has curplayer => (
+    is       => 'rw',
+    isa      => 'Games::Pandemic::Player',
+    weak_ref => 1,
+    clearer  => 'clear_curplayer',
+);
 
 # game state
 has state => ( is=>'rw', isa=>'Str' );
+has is_in_play => (
+    metaclass => 'Bool',
+    is        => 'ro',
+    isa       => 'Bool',
+    default   => 0,
+    provides  => {
+        set   => 'has_started',
+        unset => 'has_ended',
+    }
+);
 
 
 # number of research stations remaining to be build
@@ -145,7 +165,7 @@ Games::Pandemic - cooperative pandemic board game
 
 =head1 VERSION
 
-version 0.5.0
+version 0.6.0
 
 =head1 SYNOPSIS
 
